using System;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Common;
using HeuristicLab.Random;

namespace HeuristicLab.Problems.Instances.DataAnalysis {
  public class Feynman35 : FeynmanDescriptor {
    private readonly int testSamples;
    private readonly int trainingSamples;

    public Feynman35() : this((int) DateTime.Now.Ticks, 10000, 10000, null) { }

    public Feynman35(int seed) {
      Seed            = seed;
      trainingSamples = 10000;
      testSamples     = 10000;
      noiseRatio      = null;
    }

    public Feynman35(int seed, int trainingSamples, int testSamples, double? noiseRatio) {
      Seed                 = seed;
      this.trainingSamples = trainingSamples;
      this.testSamples     = testSamples;
      this.noiseRatio      = noiseRatio;
    }

    public override string Name {
      get {
        return string.Format("I.34.10 omega_0/(1-v/c) | {0}",
          noiseRatio == null ? "no noise" : string.Format(System.Globalization.CultureInfo.InvariantCulture, "noise={0:g}",noiseRatio));
      }
    }

    protected override string TargetVariable { get { return noiseRatio == null ? "omega" : "omega_noise"; } }

    protected override string[] VariableNames {
      get { return new[] {"c", "v", "omega_0", noiseRatio == null ? "omega" : "omega_noise"}; }
    }

    protected override string[] AllowedInputVariables { get { return new[] {"c", "v", "omega_0"}; } }

    public int Seed { get; private set; }

    protected override int TrainingPartitionStart { get { return 0; } }
    protected override int TrainingPartitionEnd { get { return trainingSamples; } }
    protected override int TestPartitionStart { get { return trainingSamples; } }
    protected override int TestPartitionEnd { get { return trainingSamples + testSamples; } }

    protected override List<List<double>> GenerateValues() {
      var rand = new MersenneTwister((uint) Seed);

      var data    = new List<List<double>>();
      var c       = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 3, 10).ToList();
      var v       = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 2).ToList();
      var omega_0 = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();

      var omega = new List<double>();

      data.Add(c);
      data.Add(v);
      data.Add(omega_0);
      data.Add(omega);

      for (var i = 0; i < c.Count; i++) {
        var res = omega_0[i] / (1 - v[i] / c[i]);
        omega.Add(res);
      }

      if (noiseRatio != null) {
        var omega_noise = new List<double>();
        var sigma_noise = (double) Math.Sqrt(noiseRatio.Value) * omega.StandardDeviationPop();
        omega_noise.AddRange(omega.Select(md => md + NormalDistributedRandom.NextDouble(rand, 0, sigma_noise)));
        data.Remove(omega);
        data.Add(omega_noise);
      }

      return data;
    }
  }
}